@@ -21,6 +21,14 @@ Cpanel::JSON::XS - JSON::XS for Cpanel, fast and correct serialising, also for 5
  # Note that 5.6 misses most smart utf8 and encoding functionalities
  # of newer releases.
 
+ # Note that L<JSON::MaybeXS> will automatically use Cpanel::JSON::XS
+ # if available, at virtually no speed overhead either, so you should
+ # be able to just:
+ 
+ use JSON::MaybeXS;
+
+ # and do the same things, except that you have a pure-perl fallback now.
+
 =head1 DESCRIPTION
 
 This module converts Perl data structures to JSON and vice versa. Its
@@ -42,7 +50,7 @@ values and vice versa.
 
 =over 4
 
-=item * smart Unicode handling
+=item * correct Unicode handling
 
 This module knows how to handle Unicode with Perl version higher than 5.8.5,
 documents how and when it does so, and even documents what "correct" means.
@@ -69,7 +77,7 @@ this module usually compares favourably in terms of speed, too.
 =item * simple to use
 
 This module has both a simple functional interface as well as an object
-oriented interface interface.
+oriented interface.
 
 =item * reasonably versatile output formats
 
@@ -94,7 +102,13 @@ or        L<https://rt.cpan.org/Public/Dist/Display.html?Queue=Cpanel-JSON-XS>
 
 B<Changes to JSON::XS>
 
-- public maintainance and bugtracker.
+- fixed encode of numbers for dual-vars. Different string representations
+  are preserved, but numbers with temporary strings which represent the same number
+  are here treated as numbers, not strings. Cpanel::JSON::XS is a bit slower, but
+  preserves numeric types better.
+
+- different handling of inf/nan. Default now to null, optionally with -DSTRINGIFY_INFNAN
+  to "inf"/"nan".
 
 - added C<binary> extension, non-JSON and non JSON parsable, allows
   C<\xNN> and C<\NNN> sequences.
@@ -102,12 +116,12 @@ B<Changes to JSON::XS>
 - 5.6.2 support; sacrificing some utf8 features (assuming bytes all-over),
   no multi-byte unicode characters.
 
-- use ppport.h, sanify XS.xs comment styles, harness C coding style
+- interop for true/false overloading. JSON::XS and JSON::PP representations
+  are accepted and JSON::XS accepts Cpanel::JSON::XS booleans [#13]
 
-- common::sense is optional. When available it is not used in the published
-  production module, just during development and testing.
+- ithread support. Cpanel::JSON::XS is thread-safe, JSON::XS not
 
-- extended testsuite
+- performance optimizations for threaded Perls
 
 - additional fixes for:
 
@@ -117,11 +131,21 @@ B<Changes to JSON::XS>
 
   - #7 avoid re-blessing where possible (e.g. SvREADONLY restricted hashes)
 
+- public maintenance and bugtracker
+
+- use ppport.h, sanify XS.xs comment styles, harness C coding style
+
+- common::sense is optional. When available it is not used in the published
+  production module, just during development and testing.
+
+- extended testsuite
+
+
 =cut
 
 package Cpanel::JSON::XS;
 
-our $VERSION = '2.3403';
+our $VERSION = '3.0115';
 our @ISA = qw(Exporter);
 
 our @EXPORT = qw(encode_json decode_json to_json from_json);
@@ -179,8 +203,8 @@ Except being faster.
 
 =item $is_boolean = Cpanel::JSON::XS::is_bool $scalar
 
-Returns true if the passed scalar represents either Cpanel::JSON::XS::true or
-Cpanel::JSON::XS::false, two constants that act like C<1> and C<0>, respectively
+Returns true if the passed scalar represents either C<JSON::XS::true> or
+C<JSON::XS::false>, two constants that act like C<1> and C<0>, respectively
 and are used to represent JSON C<true> and C<false> values in Perl.
 
 See MAPPING, below, for more information on how JSON values are mapped to
@@ -188,6 +212,20 @@ Perl.
 
 =back
 
+=head1 DEPRECATED FUNCTIONS
+
+=over
+
+=item from_json
+
+from_json has been renamed to decode_json
+
+=item to_json
+
+to_json has been renamed to encode_json
+
+=back
+
 
 =head1 A FEW NOTES ON UNICODE AND PERL
 
@@ -250,7 +288,7 @@ be chained:
 If C<$enable> is true (or missing), then the C<encode> method will not
 generate characters outside the code range C<0..127> (which is ASCII). Any
 Unicode characters outside that range will be escaped using either a
-single \uXXXX (BMP characters) or a double \uHHHH\uLLLLL escape sequence,
+single C<\uXXXX> (BMP characters) or a double C<\uHHHH\uLLLLL> escape sequence,
 as per RFC4627. The resulting encoded JSON text can be treated as a native
 Unicode string, an ascii-encoded, latin1-encoded or UTF-8 encoded string,
 or any other superset of ASCII.
@@ -488,6 +526,17 @@ character, after which more white-space and comments are allowed.
         # neither this one...
   ]
 
+=item * literal ASCII TAB characters in strings
+
+Literal ASCII TAB characters are now allowed in strings (and treated as
+C<\t>) in relaxed mode. Despite JSON mandates, that TAB character is
+substituted for "\t" sequence.
+
+  [
+     "Hello\tWorld",
+     "Hello<TAB>World", # literal <TAB> would not normally be allowed
+  ]
+
 =back
 
 =item $json = $json->canonical ([$enable])
@@ -499,7 +548,8 @@ by sorting their keys. This is adding a comparatively high overhead.
 
 If C<$enable> is false, then the C<encode> method will output key-value
 pairs in the order Perl stores them (which will likely change between runs
-of the same script).
+of the same script, and can change even within the same run from 5.18
+onwards).
 
 This option is useful if you want the same data structure to be encoded as
 the same JSON text (given the same overall settings). If it is disabled,
@@ -560,6 +610,8 @@ encoded. Has no effect on C<decode>.
 If C<$enable> is false (the default), then C<encode> will throw an
 exception when it encounters a blessed object.
 
+This setting has no effect on C<decode>.
+
 =item $json = $json->convert_blessed ([$enable])
 
 =item $enabled = $json->get_convert_blessed
@@ -579,12 +631,28 @@ methods called by the Perl core (== not by the user of the object) are
 usually in upper case letters and to avoid collisions with any C<to_json>
 function or method.
 
-This setting does not yet influence C<decode> in any way, but in the
-future, global hooks might get installed that influence C<decode> and are
-enabled by this setting.
+If C<$enable> is false (the default), then C<encode> will not consider
+this type of conversion.
 
-If C<$enable> is false, then the C<allow_blessed> setting will decide what
-to do when a blessed object is found.
+This setting has no effect on C<decode>.
+
+=item $json = $json->allow_tags ([$enable])
+
+=item $enabled = $json->get_allow_tags
+
+See L<OBJECT SERIALISATION> for details.
+
+If C<$enable> is true (or missing), then C<encode>, upon encountering a
+blessed object, will check for the availability of the C<FREEZE> method on
+the object's class. If found, it will be used to serialise the object into
+a nonstandard tagged JSON value (that JSON decoders cannot decode).
+
+It also causes C<decode> to parse such tagged JSON values and deserialise
+them via a call to the C<THAW> method.
+
+If C<$enable> is false (the default), then C<encode> will not consider
+this type of conversion, and tagged JSON values will cause a parse error
+in C<decode>, as if tags were not part of the grammar.
 
 =item $json = $json->filter_json_object ([$coderef->($hashref)])
 
@@ -731,6 +799,19 @@ C<0> is specified).
 
 See SECURITY CONSIDERATIONS, below, for more info on why this is useful.
 
+=item $json->stringify_infnan ([$infnan_mode = 1])
+
+=item $infnan_mode = $json->get_stringify_infnan
+
+Get or set how Cpanel::JSON::XS encodes C<inf> or C<nan> for numeric
+values. 
+
+C<null>:     infnan_mode = 0. Similar to most JSON modules in other languages.
+
+stringified: infnan_mode = 1. As in Mojo::JSON.
+
+inf/nan:     infnan_mode = 2. As in JSON::XS, and older releases. Produces invalid JSON.
+
 =item $json_text = $json->encode ($perl_scalar)
 
 Converts the given Perl data structure (a simple scalar or a reference
@@ -757,8 +838,7 @@ silently stop parsing there and return the number of characters consumed
 so far.
 
 This is useful if your JSON texts are not delimited by an outer protocol
-(which is not the brightest thing to do in the first place) and you need
-to know where the JSON text ends.
+and you need to know where the JSON text ends.
 
    Cpanel::JSON::XS->new->decode_prefix ("[1] the tail")
    => ([], 3)
@@ -815,7 +895,7 @@ If the method is called in scalar context, then it will try to extract
 exactly I<one> JSON object. If that is successful, it will return this
 object, otherwise it will return C<undef>. If there is a parse error,
 this method will croak just as C<decode> would do (one can then use
-C<incr_skip> to skip the errornous part). This is the most common way of
+C<incr_skip> to skip the erroneous part). This is the most common way of
 using the method.
 
 And finally, in list context, it will try to extract as many objects
@@ -856,7 +936,7 @@ state is left unchanged, to skip the text parsed so far and to reset the
 parse state.
 
 The difference to C<incr_reset> is that only text until the parse error
-occured is removed.
+occurred is removed.
 
 =item $json->incr_reset
 
@@ -1064,29 +1144,44 @@ it as an integer value. If that fails, it will try to represent it as
 a numeric (floating point) value if that is possible without loss of
 precision. Otherwise it will preserve the number as a string value (in
 which case you lose roundtripping ability, as the JSON number will be
-re-encoded toa JSON string).
+re-encoded to a JSON string).
 
 Numbers containing a fractional or exponential part will always be
 represented as numeric (floating point) values, possibly at a loss of
 precision (in which case you might lose perfect roundtripping ability, but
 the JSON number will still be re-encoded as a JSON number).
 
-Note that precision is not accuracy - binary floating point values cannot
-represent most decimal fractions exactly, and when converting from and to
-floating point, Cpanel::JSON::XS only guarantees precision up to but not including
-the leats significant bit.
+Note that precision is not accuracy - binary floating point values
+cannot represent most decimal fractions exactly, and when converting
+from and to floating point, C<Cpanel::JSON::XS> only guarantees precision
+up to but not including the least significant bit.
 
 =item true, false
 
-These JSON atoms become C<JSON::XS::true> and C<JSON::XS::false>,
+These JSON atoms become C<Cpanel::JSON::XS::true> and C<Cpanel::JSON::XS::false>,
 respectively. They are overloaded to act almost exactly like the numbers
 C<1> and C<0>. You can check whether a scalar is a JSON boolean by using
-the C<JSON::XS::is_bool> function.
+the C<Cpanel::JSON::XS::is_bool> function.
 
 =item null
 
 A JSON null atom becomes C<undef> in Perl.
 
+=item shell-style comments (C<< # I<text> >>)
+
+As a nonstandard extension to the JSON syntax that is enabled by the
+C<relaxed> setting, shell-style comments are allowed. They can start
+anywhere outside strings and go till the end of the line.
+
+=item tagged values (C<< (I<tag>)I<value> >>).
+
+Another nonstandard extension to the JSON syntax, enabled with the
+C<allow_tags> setting, are tagged values. In this implementation, the
+I<tag> must be a perl package/class name encoded as a JSON string, and the
+I<value> must be a JSON array encoding optional constructor arguments.
+
+See L<OBJECT SERIALISATION>, below, for details.
+
 =back
 
 
@@ -1119,7 +1214,8 @@ Perl array references become JSON arrays.
 Other unblessed references are generally not allowed and will cause an
 exception to be thrown, except for references to the integers C<0> and
 C<1>, which get turned into C<false> and C<true> atoms in JSON. You can
-also use C<JSON::XS::false> and C<JSON::XS::true> to improve readability.
+also use C<Cpanel::JSON::XS::false> and C<Cpanel::JSON::XS::true> to improve
+readability.
 
    encode_json [\0, Cpanel::JSON::XS::true]      # yields [false,true]
 
@@ -1130,8 +1226,11 @@ respectively. You can also use C<\1> and C<\0> directly if you want.
 
 =item blessed objects
 
-Blessed objects are not directly representable in JSON. See the
-C<allow_blessed> and C<convert_blessed> methods on various options on
+Blessed objects are not directly representable in JSON, but
+C<Cpanel::JSON::XS> allows various ways of handling objects. See
+L<OBJECT SERIALISATION>, below, for details.
+
+See the C<allow_blessed> and C<convert_blessed> methods on various options on
 how to deal with this: basically, you can choose between throwing an
 exception, encoding the reference as if it weren't blessed, or provide
 your own serialiser method.
@@ -1139,8 +1238,8 @@ your own serialiser method.
 =item simple scalars
 
 Simple Perl scalars (any scalar that is not a reference) are the most
-difficult objects to encode: Cpanel::JSON::XS will encode undefined scalars as
-JSON C<null> values, scalars that have last been used in a string context
+difficult objects to encode: Cpanel::JSON::XS will encode undefined scalars or inf/nan
+as JSON C<null> values, scalars that have last been used in a string context
 before encoding as JSON strings, and anything else as number value:
 
    # dump as number
@@ -1148,13 +1247,22 @@ before encoding as JSON strings, and anything else as number value:
    encode_json [-3.0e17]                # yields [-3e+17]
    my $value = 5; encode_json [$value]  # yields [5]
 
-   # used as string, so dump as string
+   # used as string, but the two representations are for the same number
    print $value;
-   encode_json [$value]                 # yields ["5"]
+   encode_json [$value]                 # yields [5]
+
+   # used as different string (non-matching dual-var)
+   my $str = '0 but true';
+   my $num = 1 + $str;
+   encode_json [$num, $str]           # yields [1,"0 but true"]
 
    # undef becomes null
    encode_json [undef]                  # yields [null]
 
+   # inf or nan becomes null, unless you answered
+   # "Do you want to handle inf/nan as strings" with yes
+   encode_json [9**9**9]                # yields [null]
+
 You can force the type to be a JSON string by stringifying it:
 
    my $x = 3.1; # some variable containing a number
@@ -1168,19 +1276,124 @@ You can force the type to be a JSON number by numifying it:
    $x += 0;     # numify it, ensuring it will be dumped as a number
    $x *= 1;     # same thing, the choice is yours.
 
-You can not currently force the type in other, less obscure, ways. Tell me
-if you need this capability (but don't forget to explain why it's needed
-:).
-
 Note that numerical precision has the same meaning as under Perl (so
 binary to decimal conversion follows the same rules as in Perl, which
 can differ to other languages). Also, your perl interpreter might expose
 extensions to the floating point numbers of your platform, such as
-infinities or NaN's - these cannot be represented in JSON, and it is an
-error to pass those in.
+infinities or NaN's - these cannot be represented in JSON, and thus
+null is returned instead. Optionally you can configure it to stringify
+inf and nan values.
 
 =back
 
+=head2 OBJECT SERIALISATION
+
+As JSON cannot directly represent Perl objects, you have to choose between
+a pure JSON representation (without the ability to deserialise the object
+automatically again), and a nonstandard extension to the JSON syntax,
+tagged values.
+
+=head3 SERIALISATION
+
+What happens when C<Cpanel::JSON::XS> encounters a Perl object depends on the
+C<allow_blessed>, C<convert_blessed> and C<allow_tags> settings, which are
+used in this order:
+
+=over 4
+
+=item 1. C<allow_tags> is enabled and the object has a C<FREEZE> method.
+
+In this case, C<Cpanel::JSON::XS> uses the L<Types::Serialiser> object
+serialisation protocol to create a tagged JSON value, using a nonstandard
+extension to the JSON syntax.
+
+This works by invoking the C<FREEZE> method on the object, with the first
+argument being the object to serialise, and the second argument being the
+constant string C<JSON> to distinguish it from other serialisers.
+
+The C<FREEZE> method can return any number of values (i.e. zero or
+more). These values and the paclkage/classname of the object will then be
+encoded as a tagged JSON value in the following format:
+
+   ("classname")[FREEZE return values...]
+
+e.g.:
+
+   ("URI")["http://www.google.com/"]
+   ("MyDate")[2013,10,29]
+   ("ImageData::JPEG")["Z3...VlCg=="]
+
+For example, the hypothetical C<My::Object> C<FREEZE> method might use the
+objects C<type> and C<id> members to encode the object:
+
+   sub My::Object::FREEZE {
+      my ($self, $serialiser) = @_;
+
+      ($self->{type}, $self->{id})
+   }
+
+=item 2. C<convert_blessed> is enabled and the object has a C<TO_JSON> method.
+
+In this case, the C<TO_JSON> method of the object is invoked in scalar
+context. It must return a single scalar that can be directly encoded into
+JSON. This scalar replaces the object in the JSON text.
+
+For example, the following C<TO_JSON> method will convert all L<URI>
+objects to JSON strings when serialised. The fatc that these values
+originally were L<URI> objects is lost.
+
+   sub URI::TO_JSON {
+      my ($uri) = @_;
+      $uri->as_string
+   }
+
+=item 3. C<allow_blessed> is enabled.
+
+The object will be serialised as a JSON null value.
+
+=item 4. none of the above
+
+If none of the settings are enabled or the respective methods are missing,
+C<Cpanel::JSON::XS> throws an exception.
+
+=back
+
+=head3 DESERIALISATION
+
+For deserialisation there are only two cases to consider: either
+nonstandard tagging was used, in which case C<allow_tags> decides,
+or objects cannot be automatically be deserialised, in which
+case you can use postprocessing or the C<filter_json_object> or
+C<filter_json_single_key_object> callbacks to get some real objects our of
+your JSON.
+
+This section only considers the tagged value case: I a tagged JSON object
+is encountered during decoding and C<allow_tags> is disabled, a parse
+error will result (as if tagged values were not part of the grammar).
+
+If C<allow_tags> is enabled, C<Cpanel::JSON::XS> will look up the C<THAW> method
+of the package/classname used during serialisation (it will not attempt
+to load the package as a Perl module). If there is no such method, the
+decoding will fail with an error.
+
+Otherwise, the C<THAW> method is invoked with the classname as first
+argument, the constant string C<JSON> as second argument, and all the
+values from the JSON array (the values originally returned by the
+C<FREEZE> method) as remaining arguments.
+
+The method must then return the object. While technically you can return
+any Perl scalar, you might have to enable the C<enable_nonref> setting to
+make that work in all cases, so better return an actual blessed reference.
+
+As an example, let's implement a C<THAW> function that regenerates the
+C<My::Object> from the C<FREEZE> example earlier:
+
+   sub My::Object::THAW {
+      my ($class, $serialiser, $type, $id) = @_;
+
+      $class->new (type => $type, id => $id)
+   }
+
 
 =head1 ENCODING/CODESET FLAG NOTES
 
@@ -1215,7 +1428,7 @@ the same time, which can be confusing.
 When C<utf8> is disabled (the default), then C<encode>/C<decode> generate
 and expect Unicode strings, that is, characters with high ordinal Unicode
 values (> 255) will be encoded as such characters, and likewise such
-characters are decoded as-is, no canges to them will be done, except
+characters are decoded as-is, no changes to them will be done, except
 "(re-)interpreting" them as Unicode codepoints or Unicode characters,
 respectively (to Perl, these are the same thing in strings unless you do
 funny/weird/dumb stuff).
@@ -1344,69 +1557,24 @@ output for these property strings, e.g.:
    $json =~ s/"__proto__"\s*:/"__proto__renamed":/g;
 
 This works because C<__proto__> is not valid outside of strings, so every
-occurence of C<"__proto__"\s*:> must be a string used as property name.
+occurrence of C<"__proto__"\s*:> must be a string used as property name.
 
 If you know of other incompatibilities, please let me know.
 
 
 =head2 JSON and YAML
 
-You often hear that JSON is a subset of YAML. This is, however, a mass
-hysteria(*) and very far from the truth (as of the time of this writing),
-so let me state it clearly: I<in general, there is no way to configure
-JSON::XS to output a data structure as valid YAML> that works in all
-cases.
-
-If you really must use Cpanel::JSON::XS to generate YAML, you should use this
-algorithm (subject to change in future versions):
+You often hear that JSON is a subset of YAML.  I<in general, there is
+no way to configure JSON::XS to output a data structure as valid YAML>
+that works in all cases.  If you really must use Cpanel::JSON::XS to
+generate YAML, you should use this algorithm (subject to change in
+future versions):
 
    my $to_yaml = Cpanel::JSON::XS->new->utf8->space_after (1);
    my $yaml = $to_yaml->encode ($ref) . "\n";
 
 This will I<usually> generate JSON texts that also parse as valid
-YAML. Please note that YAML has hardcoded limits on (simple) object key
-lengths that JSON doesn't have and also has different and incompatible
-unicode character escape syntax, so you should make sure that your hash
-keys are noticeably shorter than the 1024 "stream characters" YAML allows
-and that you do not have characters with codepoint values outside the
-Unicode BMP (basic multilingual page). YAML also does not allow C<\/>
-sequences in strings (which Cpanel::JSON::XS does not I<currently> generate, but
-other JSON generators might).
-
-There might be other incompatibilities that I am not aware of (or the YAML
-specification has been changed yet again - it does so quite often). In
-general you should not try to generate YAML with a JSON generator or vice
-versa, or try to parse JSON with a YAML parser or vice versa: chances are
-high that you will run into severe interoperability problems when you
-least expect it.
-
-=over 4
-
-=item (*)
-
-I have been pressured multiple times by Brian Ingerson (one of the
-authors of the YAML specification) to remove this paragraph, despite him
-acknowledging that the actual incompatibilities exist. As I was personally
-bitten by this "JSON is YAML" lie, I refused and said I will continue to
-educate people about these issues, so others do not run into the same
-problem again and again. After this, Brian called me a (quote)I<complete
-and worthless idiot>(unquote).
-
-In my opinion, instead of pressuring and insulting people who actually
-clarify issues with YAML and the wrong statements of some of its
-proponents, I would kindly suggest reading the JSON spec (which is not
-that difficult or long) and finally make YAML compatible to it, and
-educating users about the changes, instead of spreading lies about the
-real compatibility for many I<years> and trying to silence people who
-point out that it isn't true.
-
-Addendum/2009: the YAML 1.2 spec is still incompatible with JSON, even
-though the incompatibilities have been documented (and are known to Brian)
-for many years and the spec makes explicit claims that YAML is a superset
-of JSON. It would be so easy to fix, but apparently, bullying people and
-corrupting userdata is so much easier.
-
-=back
+YAML.
 
 
 =head2 SPEED
@@ -1416,11 +1584,11 @@ tables. They have been generated with the help of the C<eg/bench> program
 in the JSON::XS distribution, to make it easy to compare on your own
 system.
 
-JSON::XS is with L<Data::MessagePack> one of the fastest serializers,
-because JSON and JSON::XS do not support backrefs (no graph structures),
-only trees. Storable supports backrefs, i.e. graphs. Data::MessagePack
-encodes its data binary (as Storable) and supports only very simple
-subset of JSON.
+JSON::XS is with L<Data::MessagePack> and L<Sereal> one of the fastest
+serializers, because JSON and JSON::XS do not support backrefs (no
+graph structures), only trees. Storable supports backrefs,
+i.e. graphs. Data::MessagePack encodes its data binary (as Storable)
+and supports only very simple subset of JSON.
 
 First comes a comparison between various modules using
 a very short single-line JSON string (also available at
@@ -1477,6 +1645,23 @@ will be broken due to missing (or wrong) Unicode handling. Others refuse
 to decode or encode properly, so it was impossible to prepare a fair
 comparison table for that case.
 
+For updated graphs see L<https://github.com/Sereal/Sereal/wiki/Sereal-Comparison-Graphs>
+
+
+=head1 INTEROP with JSON and JSON::XS
+
+JSON-XS-3.01 broke interoperability with JSON-2.90 with booleans. See L<JSON>.
+
+Cpanel::JSON::XS needs to know the JSON and JSON::XS versions to be able work
+with those objects, especially when encoding a booleans like C<{"is_true":true}>.
+So you need to load these modules before.
+
+true/false overloading is supported.
+
+JSON::XS and JSON::PP representations are accepted and older JSON::XS accepts
+Cpanel::JSON::XS booleans.
+
+I cannot think of any reason to still use JSON::XS anymore.
 
 =head1 SECURITY CONSIDERATIONS
 
@@ -1484,8 +1669,7 @@ When you are using JSON in a protocol, talking to untrusted potentially
 hostile creatures requires relatively few measures.
 
 First of all, your JSON decoder should be secure, that is, should not have
-any buffer overflows. Obviously, this module should ensure that and I am
-trying hard on making that true, but you never know.
+any buffer overflows. Obviously, this module should ensure that.
 
 Second, you need to avoid resource-starving attacks. That means you should
 limit the size of JSON texts you accept, or make sure then when your
@@ -1496,7 +1680,7 @@ it into a Perl structure. While JSON::XS can check the size of the JSON
 text, it might be too late when you already have it in memory, so you
 might want to check the size before you accept the string.
 
-Third, JSON::XS recurses using the C stack when decoding objects and
+Third, Cpanel::JSON::XS recurses using the C stack when decoding objects and
 arrays. The C stack is a limited resource: for instance, on my amd64
 machine with 8MB of stack size I can decode around 180k nested arrays but
 only 14k nested JSON objects (due to perl itself recursing deeply on croak
@@ -1505,37 +1689,28 @@ conservative, the default nesting limit is set to 512. If your process
 has a smaller stack, you should adjust this setting accordingly with the
 C<max_depth> method.
 
-Something else could bomb you, too, that I forgot to think of. In that
-case, you get to keep the pieces. I am always open for hints, though...
-
-Also keep in mind that JSON::XS might leak contents of your Perl data
+Also keep in mind that Cpanel::JSON::XS might leak contents of your Perl data
 structures in its error messages, so when you serialise sensitive
 information you might want to make sure that exceptions thrown by JSON::XS
 will not end up in front of untrusted eyes.
 
-If you are using JSON::XS to return packets to consumption
+If you are using Cpanel::JSON::XS to return packets to consumption
 by JavaScript scripts in a browser you should have a look at
 L<http://blog.archive.jpsykes.com/47/practical-csrf-and-json-security/> to
 see whether you are vulnerable to some common attack vectors (which really
 are browser design bugs, but it is still you who will have to deal with
 it, as major browser developers care only for features, not about getting
-security right).
-
+security right). You might also want to also look at L<Mojo::JSON>
+special escape rules to prevent from XSS attacks.
 
 =head1 THREADS
 
-This module is I<not> guaranteed to be thread safe and there are no
-plans to change this until Perl gets thread support (as opposed to the
-horribly slow so-called "threads" which are simply slow and bloated
-process simulations - use fork, it's I<much> faster, cheaper, better).
-
-(It might actually work, but you have been warned).
-
-L<Data::MessagePack> in comparison is thread-safe.
+Cpanel::JSON::XS has proper ithreads support, unlike JSON::XS. If you
+encounter any bugs with thread support please report them.
 
 =head1 BUGS
 
-While the goal of the JSON::XS module is to be correct, that
+While the goal of the Cpanel::JSON::XS module is to be correct, that
 unfortunately does not mean it's bug-free, only that the author thinks
 its design is bug-free. If you keep reporting bugs they will be fixed
 swiftly, though.
@@ -1546,41 +1721,53 @@ to report any issues twice. Once in private to MLEHMANN to be fixed in
 JSON::XS for the masses and one to our the public tracker. Issues
 fixed by JSON::XS with a new release will also be backported to
 Cpanel::JSON::XS and 5.6.2, as long as Cpanel relies on 5.6.2 and
-JSON::XS as our serializer of choice.
+Cpanel::JSON::XS as our serializer of choice.
 
 L<https://rt.cpan.org/Public/Dist/Display.html?Queue=Cpanel-JSON-XS>
 
+=head1 LICENSE
+
+This module is available under the same licences as perl, the Artistic
+license and the GPL.
+
 =cut
 
-our $true  = do { bless \(my $dummy = 1), "Cpanel::JSON::XS::Boolean" };
-our $false = do { bless \(my $dummy = 0), "Cpanel::JSON::XS::Boolean" };
+our ($true, $false);
+if ($INC{'JSON/XS.pm'} and $JSON::XS::VERSION ge "3.00") {
+  $true  = $Types::Serialiser::true; # readonly if loaded by JSON::XS
+  $false = $Types::Serialiser::false;
+} else {
+  $true  = do { bless \(my $dummy = 1), "JSON::XS::Boolean" };
+  $false = do { bless \(my $dummy = 0), "JSON::XS::Boolean" };
+}
 
 sub true()  { $true  }
 sub false() { $false }
 
 sub is_bool($) {
-   UNIVERSAL::isa $_[0], "Cpanel::JSON::XS::Boolean"
-#      or UNIVERSAL::isa $_[0], "JSON::Literal"
+  UNIVERSAL::isa($_[0], "JSON::XS::Boolean")
+    or UNIVERSAL::isa($_[0], "JSON::PP::Boolean");
 }
 
 XSLoader::load 'Cpanel::JSON::XS', $VERSION;
 
-package Cpanel::JSON::XS::Boolean;
+package
+  JSON::XS::Boolean;
 
 use overload
    "0+"     => sub { ${$_[0]} },
    "++"     => sub { $_[0] = ${$_[0]} + 1 },
    "--"     => sub { $_[0] = ${$_[0]} - 1 },
-  '""'      => sub { ${$_[0]} == 1 ? 'true' : 'false' },
-  'eq'      => sub {
-    my ($obj, $op) = ref ($_[0]) ? ($_[0], $_[1]) : ($_[1], $_[0]);
-    if ($op eq 'true' or $op eq 'false') {
-      return "$obj" eq 'true' ? 'true' eq $op : 'false' eq $op;
-    }
-    else {
-      return $obj ? 1 == $op : 0 == $op;
-    }
-   },
+  # '""'    => sub { ${$_[0]} == 1 ? 'true' : 'false' },
+  #'eq'      => sub {
+  #  my ($obj, $op) = ref ($_[0]) ? ($_[0], $_[1]) : ($_[1], $_[0]);
+  #  if ($op eq 'true' or $op eq 'false') {
+  #    return "$obj" eq 'true' ? 'true' eq $op : 'false' eq $op;
+  #  }
+  #  else {
+  #    return $obj ? 1 == $op : 0 == $op;
+  #  }
+  # },
    fallback => 1;
 
 1;
@@ -1589,10 +1776,14 @@ use overload
 
 The F<cpanel_json_xs> command line utility for quick experiments.
 
+L<JSON>, L<JSON::XS>, L<JSON::MaybeXS>, L<Mojo::JSON>, L<Mojo::JSON::MaybeXS>,
+L<JSON::SL>, L<JSON::DWIW>, L<JSON::YAJL>, L<https://metacpan.org/search?q=JSON>
+
 =head1 AUTHOR
 
-  Marc Lehmann <schmorp@schmorp.de>
-  http://home.schmorp.de/
+  Marc Lehmann <schmorp@schmorp.de>, http://home.schmorp.de/
+
+  cPanel Inc. <cpan@cpanel.net>
 
 =head1 MAINTAINER
 
